﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

namespace Util
{
	class UDPserver
	{
		private int PORT_NUMBER = 9901;

		private UdpClient udp = null;
		IAsyncResult ar_ = null;


		public void Start(int port)
		{
			udp = new UdpClient();

			PORT_NUMBER = port;

			OnReceiveMessage += new UDPserver.ReceiveMessageHandler(udp_OnReceiveMessage);

			IPEndPoint localEp = new IPEndPoint(IPAddress.Any, PORT_NUMBER);

			udp.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
			udp.ExclusiveAddressUse = false;
			udp.Client.Bind(localEp);

			StartListening();
		}



		public void Stop()
		{
			try
			{
				if (udp != null)
				{
					udp.Close();
					udp = null;
				}
			}
			catch { }
		}



		private void StartListening()
		{
			ar_ = udp.BeginReceive(Receive, new object());
		}



		public void Receive(IAsyncResult ar)
		{
			try
			{
				if (udp != null)
				{
					IPEndPoint ip = new IPEndPoint(IPAddress.Any, PORT_NUMBER);
					byte[] bytes = udp.EndReceive(ar, ref ip);

					string message = Encoding.UTF8.GetString(bytes);
					string ipstr = ip.Address.ToString();

					//dbg.AddMsg(ipstr);
					//dbg.AddMsg(message);
					StartListening();

					if (OnReceiveMessage != null) OnReceiveMessage(message);
				}
			}
			catch (SocketException se)
			{
				//dbg.AddMsg("SocketException :" + se.ToString());
			}
			catch (Exception ex)
			{
				//dbg.AddMsg("Exception : " + ex.ToString());
			}
		}



		void udp_OnReceiveMessage(string message)
		{
			NetMsg.Add(message);
		}

		public delegate void ReceiveMessageHandler(string message);
		public event ReceiveMessageHandler OnReceiveMessage;
	}



	static public class NetMsg
	{
		const int MAX_LINES = 10000;
		static private string[] msg = new string[MAX_LINES];
		static private int Wpos = 0;
		static private int Rpos = 0;

		static public bool Relay = false;



		static public void Add(string str)
		{
			int t;
			t = Wpos;
			t++;
			if (t > MAX_LINES - 1) t = 0;
			if (t == Rpos) return;
			msg[t] = str;
			Wpos = t;
		}



		static public string GetMsg()
		{
			if (Rpos == Wpos) return "";
			Rpos++;
			if (Rpos > MAX_LINES - 1) Rpos = 0;
			return msg[Rpos];
		}
	}
}

